//-------------------------------------------------------------------
// File    : ub_xnet_l0.h
// Author  : Uwe Becker
// Date    : 2018-01-14
//
// Project : XNET
// Modul   : Protocol layer L0 [hardware layer]
// MCU and hardware related functions
//-------------------------------------------------------------------

#ifndef UB_XNET_L0_H_
#define UB_XNET_L0_H_


//-------------------------------------------------------------------
// to port this project to another hardware:
// change these parameter related to the hardware of your node
//-------------------------------------------------------------------

#define NODE_PORT_CNT	3 // number of physical ports on this node

#define NODE_TX_BUFFER_SIZE		5	// max number of tx frames in the send queue
#define NODE_RX_BUFFER_SIZE		10	// max number of rx frames in the receive queue

// type number for XNET hardware implementation (8bit)
//#define XNET_VERSION_HW_8BIT			0x01	// 0x01 = STM32F103C8T6
#define XNET_VERSION_HW_8BIT			0x02	// 0x02 = STM32F407VGT6

//-------------------------------------------------------------------
// don't make changes below this line
//-------------------------------------------------------------------

// version number for XNET layer-0 (8bit)
#define XNET_VERSION_L0_8BIT			0x02	// 0x02 = version 0.2


// include global header
#include "ub_xnet_global.h"


#if NODE_PORT_CNT == 0
  #error NODE_PORT_CNT: node must have at least one port
#endif

#if (NODE_PORT_CNT > XNET_MAX_PORTS) && (XNET_MAX_PORTS > 0)
  #error NODE_PORT_CNT: too many ports on this node
#endif


// delay for hello packages
#define NODE_HELLO_OFFSET		(XNET_HELLO_DELAY_INTERVAL_ms / NODE_PORT_CNT)
#define NODE_HELLO_DELAY_SKIP	(XNET_HELLO_DELAY_INTERVAL_ms + (XNET_HELLO_DELAY_INTERVAL_ms / 2))



// function calls for layer-1
void UB_Xnet_L0_Init(void);
void UB_Xnet_L0_Delay_ms(U32_DWORD delay_ms);
// uart functions
U8_BOOL UB_Xnet_L0_Check_Rx_Buf(U8_CHAR uart_port);
U8_CHAR UB_Xnet_L0_Read_Rx_Buf(U8_CHAR uart_port);
void UB_Xnet_L0_Send_Byte(U8_CHAR uart_port, U8_CHAR tx_value);
void UB_Xnet_L0_Send_Array(U8_CHAR uart_port, U8_CHAR *ptr, U8_CHAR size);
void UB_Xnet_L0_Clear_Rx_Ringbuffer(U8_CHAR uart_port);
U8_CHAR UB_Xnet_L0_Get_Rx_Ringbuffer_Cnt(U8_CHAR uart_port);
U8_CHAR UB_Xnet_L0_Read_Rx_Ringbuffer(U8_CHAR uart_port);
// status LED functions
void UB_Xnet_L0_Status_Led_On(void);
void UB_Xnet_L0_Status_Led_Off(void);




#endif /* UB_XNET_L0_H_ */
