//-------------------------------------------------------------------
// File    : ub_xnet_l2.h
// Author  : Uwe Becker
// Date    : 2018-01-14
//
// Project : XNET
// Modul   : Protocol layer L2 [network layer]
// Network handling and User API's (for application layer)
//-------------------------------------------------------------------

#ifndef UB_XNET_L2_H_
#define UB_XNET_L2_H_





//-------------------------------------------------------------------
// don't make changes below this line
// to port this project to another MCU or hardware:
// change "xnet_l0.c", "xnet_l0.h", "xnet_global.h"
//-------------------------------------------------------------------


// version number for XNET layer-2 (8bit)
#define XNET_VERSION_L2_8BIT			0x04	// 0x04 = version 0.4




// include xnet layer-1 and global header
#include "ub_xnet_l1.h"
#include "ub_xnet_global.h"

typedef enum {
	CMD_TYPE_EMPTY = 0,
	CMD_TYPE_UNKNOWN,
	CMD_TYPE_DEBUG_ANSWER,
	CMD_TYPE_NODE_DATA,
	CMD_TYPE_GROUP_DATA,
	CMD_TYPE_DATA_ANSWER
}CmdType_t;

typedef enum {
	NETINFO_NR_NODES = 0,
	NETINFO_NR_NEIGHBORS,
	NETINFO_OWN_NODE_ID,
	NETINFO_OWN_GROUP_MASK,
	NOTINFO_VERSION_NR
}NetInfo_t;


typedef struct {
	U8_CHAR port_nr;
	U8_CHAR hop_cnt;
}PathTable_t;

typedef struct {
	U8_CHAR rd_ptr;
	U8_CHAR wr_ptr;
	Payload_t payload[NODE_RX_BUFFER_SIZE];
}RXBuf_t;

typedef struct {
	U8_BOOL path_table_ok;
	U8_BOOL new_neighbor;
	U8_CHAR neighbor_port;
	U8_BOOL flooding_table;
	U8_CHAR flood_ls_node_idx;
}State_t;

typedef struct {
	U8_CHAR nr_nodes;					// number of connected nodes in the network
	U8_CHAR nr_neighbors;				// number of direct connected neighbors to the node
}Connection_t;

typedef struct {
	U8_BOOL running;
	U8_BOOL pong_ok;
	U32_DWORD start_time;
	U32_DWORD stop_time;
}Ping_t;


typedef struct {
	U8_CHAR Node_Id_List[XNET_MAX_NODES];				// NodeIdList of the node
	Connection_t connection;
	U8_CHAR Link_State_Table[XNET_MAX_NODES][XNET_MAX_PORTS]; // LinkStateTable of the network
	PathTable_t Path_Table[XNET_MAX_NODES];
	State_t state;
	U8_CHAR group_mask;
	RXBuf_t rx_buf;
	Ping_t ping;
}XNet_t;
XNet_t XNet;



// user API calls
U8_BOOL UB_Xnet_L2_Init(U8_CHAR own_id);
void UB_Xnet_L2_1ms_Tick(void);
void UB_Xnet_L2_Loop(void);
U32_DWORD UB_Xnet_L2_Get_NetInfo(NetInfo_t mode);
U8_CHAR UB_Xnet_L2_Get_Node_ID(U8_CHAR node_index);
U8_CHAR UB_Xnet_L2_Get_Node_Hops(U8_CHAR node_index);
void UB_Xnet_L2_Set_Group_Mask(U8_CHAR group_mask);
U8_BOOL UB_Xnet_L2_Send_Debug_To_Node(U8_CHAR destination_id, U8_CHAR dbg_mode, U8_CHAR *data_ptr, U8_CHAR data_size);
U8_BOOL UB_Xnet_L2_Send_Debug_To_All(U8_CHAR dbg_mode, U8_CHAR *data_ptr, U8_CHAR data_size);
U8_BOOL UB_Xnet_L2_Send_Data_To_Node(U8_CHAR destination_id, U8_CHAR data_mode, U8_CHAR *data_ptr, U8_CHAR data_size);
U8_BOOL UB_Xnet_L2_Send_Data_To_Group(U8_CHAR group_flags, U8_CHAR data_mode, U8_CHAR *data_ptr, U8_CHAR data_size);
U8_BOOL UB_Xnet_L2_Send_Answer_To_Node(U8_CHAR destination_id, U8_CHAR data_mode, U8_CHAR *data_ptr, U8_CHAR data_size);
U8_BOOL UB_Xnet_L2_Check_Rx_Buffer(void);
CmdType_t UB_Xnet_L2_Read_Rx_Buffer(U8_CHAR *source_id, U8_CHAR *mode, U8_CHAR *data_ptr, U8_CHAR *data_size);
U32_DWORD UB_Xnet_L2_Get_Tic(void);
U8_BOOL UB_Xnet_L2_Start_Ping_Node(U8_CHAR destination_id);


#endif /* UB_XNET_L2_H_ */
