//-------------------------------------------------------------------
// File    : stm32_ub_system.c
// Author  : Uwe Becker
// Date    : 2018-01-14
// Version : 0.01
//
// Project : STM32
// Modul   : SYSTEM-Library
// System-Library for STM32F407
//-------------------------------------------------------------------

#include "stm32_ub_system.h"

void UB_System_Init(void)
{
	  RCC_ClkInitTypeDef RCC_ClkInitStruct;
	  RCC_OscInitTypeDef RCC_OscInitStruct;

	  /* Enable Power Control clock */
	  __HAL_RCC_PWR_CLK_ENABLE();

	  /* The voltage scaling allows optimizing the power consumption when the device is
	     clocked below the maximum system frequency, to update the voltage scaling value
	     regarding system frequency refer to product datasheet.  */
	  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

	  /* Enable HSE Oscillator and activate PLL with HSE as source */
	  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
	  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
	  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
	  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
	  RCC_OscInitStruct.PLL.PLLM = 8;
	  RCC_OscInitStruct.PLL.PLLN = 336;
	  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
	  RCC_OscInitStruct.PLL.PLLQ = 7;
	  if(HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
	  {
		  while (1);
	  }

	  /* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2
	     clocks dividers */
	  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
	  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
	  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
	  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;
	  if(HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_5) != HAL_OK)
	  {
		  while (1);
	  }

	  /* STM32F405x/407x/415x/417x Revision Z devices: prefetch is supported  */
	  if (HAL_GetREVID() == 0x1001)
	  {
	    /* Enable the Flash prefetch */
	    __HAL_FLASH_PREFETCH_BUFFER_ENABLE();
	  }
}


void UB_System_GpioClockEnable(GPIO_TypeDef *port)
{
	if(port == GPIOA)  __HAL_RCC_GPIOA_CLK_ENABLE();
	if(port == GPIOB)  __HAL_RCC_GPIOB_CLK_ENABLE();
	if(port == GPIOC)  __HAL_RCC_GPIOC_CLK_ENABLE();
	if(port == GPIOD)  __HAL_RCC_GPIOD_CLK_ENABLE();
	if(port == GPIOE)  __HAL_RCC_GPIOE_CLK_ENABLE();
}

void UB_System_UartClockEnable(USART_TypeDef *port)
{
	if(port == USART1) __HAL_RCC_USART1_CLK_ENABLE();
	if(port == USART2) __HAL_RCC_USART2_CLK_ENABLE();
	if(port == USART3) __HAL_RCC_USART3_CLK_ENABLE();
	if(port == UART4) __HAL_RCC_UART4_CLK_ENABLE();
	if(port == UART5) __HAL_RCC_UART5_CLK_ENABLE();
	if(port == USART6) __HAL_RCC_USART6_CLK_ENABLE();
}

void UB_System_AFIORemapUart(USART_TypeDef *uart_port, GPIO_TypeDef *gpio_port)
{
	//n.c.
}

void UB_System_AFIOClockEnable(void)
{
	//n.c.
}


void UB_System_Delay_ms(uint32_t delay)
{
	HAL_Delay(delay);
}

uint32_t UB_System_Get_UID(void)
{
	uint32_t value = 0;
	uint32_t buf[3];

	HAL_GetUID(buf);

	value = buf[0];

	return value;
}
